/*
To: Roger Kraft <roger@calumet.purdue.edu>
CC: Jerry Dekker <jerrydekker@comcast.net>
Subject: Re: portable MPICH

Professor Kraft:

Neat stuff!!
I messed around with it a bit, and came up with the idea
of putting the batch file stuff in a header file.

I attached it if you're interested.

To use it:
In the program source code--
change references against "mpi.h" to "mpi_local.h";
and "MPI_Init(&argc,&argv) to MPI_Init_local(&argc,&argv).

The compiled programs will then use the
root node to spawn the worker nodes, the
number of which is controlled by the parameter "-n",
e.g. "cpi.exe -n 4".

Jerry
*/
#ifndef _MPI_LOCAL_H
#define _MPI_LOCAL_H

#include <stdlib.h>
#include <ctype.h>
#include <stdio.h>

#define WINDOWS_LEAN_AND_MEAN
#include <windows.h>
#define snprintf _snprintf

#include "mpi.h"

#define MAX_ARG_STR 1024
#define MAX_TMP_STR  256

int MPI_Init_local(int *argc, char ***argv)
{
   if (!argc || !argv) {
      printf(
         "MPI_Init_local: Expected argument parameters "
         "but encountered null pointer(s). Exiting...\n");
      exit(1);
   }

   if (!getenv("FIRST_MPI_NODE_DEFINED")) {
      int count, ret_value, number = 1;
      char temp[MAX_TMP_STR], arg_string[MAX_ARG_STR];
      STARTUPINFO start_info;
      PROCESS_INFORMATION process_info;
      memset(&start_info,0,sizeof(STARTUPINFO));

      /* Only the first process (root node) will ever get here... */
      putenv("FIRST_MPI_NODE_DEFINED=TRUE");

      /* Program arguments: find 'n' node indicator,concatenate the rest... */
      strncpy(arg_string,(*argv)[0],MAX_ARG_STR);
      count = 1;
      while (count < *argc) {
         /* If a valid node count parameter: use it, but don't pass it on... */
         if (!strcmp((*argv)[count],"-n")) {
            if ((*argv)[count + 1] && isdigit((*argv)[count + 1][0])) {
               number = atoi((*argv)[count + 1]);
               if (!number) number = 1; /* "-n 0" was encountered. */
               count += 2;
               continue;
            }
         }
         /* ...otherwise, just add it to the process creation string. */
         snprintf(arg_string,MAX_ARG_STR,"%s %s",arg_string,(*argv)[count]);
         count++;
      }

      /* Set up the common environmental variables... */
      putenv("PMI_ROOT_HOST=localhost");
      putenv("PMI_ROOT_PORT=9222");
      putenv("PMI_ROOT_LOCAL=1");
      snprintf(temp,MAX_TMP_STR,"PMI_SIZE=%d",number);
      putenv(temp);

      /* Launch the MPI node processes... */
      count = 1;
      while (count < number) {
         /* Set the new process' rank... */
         snprintf(temp,MAX_TMP_STR,"PMI_RANK=%d",count);
         putenv(temp);

         ret_value = CreateProcess(
            NULL,
            arg_string,
            0,
            0,
            FALSE,
            CREATE_DEFAULT_ERROR_MODE | DETACHED_PROCESS,
            0,
            0,
            &start_info,
            &process_info);
         if (!ret_value)
            printf("MPI_Init_local: Node %d could not be spawned.\n",count);
         count++;
      }
      /* Set the rank for this process (the root node)... */
      putenv("PMI_RANK=0");
   }
   /* Make the call to the MPI initializer. */
   return MPI_Init(argc,argv);
}

#endif
